@@ -289,7 +289,7 @@ pub fn get_polygon_normal(polygon: &[Vector3<f32>]) -> Result<Vector3<f32>, &'st
     }
 
     normal
-        .try_normalize(std::f32::EPSILON)
+        .try_normalize(f32::EPSILON)
         .ok_or("Unable to get normal of degenerated polygon!")
 }
 
@@ -404,9 +404,47 @@ pub fn lerpf(a: f32, b: f32, t: f32) -> f32 {
     a + (b - a) * t
 }
 
+// https://en.wikipedia.org/wiki/Cubic_Hermite_spline
+pub fn cubicf(p0: f32, p1: f32, t: f32, m0: f32, m1: f32) -> f32 {
+    let t2 = t * t;
+    let t3 = t2 * t;
+    let scale = (p1 - p0).abs();
+
+    (2.0 * t3 - 3.0 * t2 + 1.0) * p0
+        + (t3 - 2.0 * t2 + t) * m0 * scale
+        + (-2.0 * t3 + 3.0 * t2) * p1
+        + (t3 - t2) * m1 * scale
+}
+
+pub fn cubicf_derivative(p0: f32, p1: f32, t: f32, m0: f32, m1: f32) -> f32 {
+    let t2 = t * t;
+    let scale = (p1 - p0).abs();
+
+    (6.0 * t2 - 6.0 * t) * p0
+        + (3.0 * t2 - 4.0 * t + 1.0) * m0 * scale
+        + (6.0 * t - 6.0 * t2) * p1
+        + (3.0 * t2 - 2.0 * t) * m1 * scale
+}
+
+pub fn inf_sup_cubicf(p0: f32, p1: f32, m0: f32, m1: f32) -> (f32, f32) {
+    // Find two `t`s where derivative of cubicf is zero - these will be
+    // extreme points of the spline. Then get the values at those `t`s
+    let d = -(9.0 * p0 * p0 + 6.0 * p0 * (-3.0 * p1 + m1 + m0) + 9.0 * p1 * p1
+        - 6.0 * p1 * (m1 + m0)
+        + m1 * m1
+        + m1 * m0
+        + m0 * m0)
+        .sqrt();
+    let k = 3.0 * (2.0 * p0 - 2.0 * p1 + m1 + m0);
+    let v = 3.0 * p0 - 3.0 * p1 + m1 + 2.0 * m0;
+    let t0 = (-d + v) / k;
+    let t1 = (d + v) / k;
+    (cubicf(p0, p1, t0, m0, m1), cubicf(p0, p1, t1, m0, m1))
+}
+
 pub fn get_farthest_point(points: &[Vector3<f32>], dir: Vector3<f32>) -> Vector3<f32> {
     let mut n_farthest = 0;
-    let mut max_dot = -std::f32::MAX;
+    let mut max_dot = -f32::MAX;
     for (i, point) in points.iter().enumerate() {
         let dot = dir.dot(point);
         if dot > max_dot {
@@ -665,7 +703,7 @@ impl AsMut<[u32]> for TriangleDefinition {
 ///
 /// O(n) complexity.
 pub fn get_closest_point<P: PositionProvider>(points: &[P], point: Vector3<f32>) -> Option<usize> {
-    let mut closest_sqr_distance = std::f32::MAX;
+    let mut closest_sqr_distance = f32::MAX;
     let mut closest_index = None;
     for (i, vertex) in points.iter().enumerate() {
         let sqr_distance = (vertex.position() - point).norm_squared();
@@ -683,7 +721,7 @@ pub fn get_closest_point_triangles<P: PositionProvider>(
     triangle_indices: &[u32],
     point: Vector3<f32>,
 ) -> Option<usize> {
-    let mut closest_sqr_distance = std::f32::MAX;
+    let mut closest_sqr_distance = f32::MAX;
     let mut closest_index = None;
     for triangle_index in triangle_indices {
         let triangle = triangles.get(*triangle_index as usize).unwrap();
@@ -704,7 +742,7 @@ pub fn get_closest_point_triangle_set<P: PositionProvider>(
     triangles: &[TriangleDefinition],
     point: Vector3<f32>,
 ) -> Option<usize> {
-    let mut closest_sqr_distance = std::f32::MAX;
+    let mut closest_sqr_distance = f32::MAX;
     let mut closest_index = None;
     for triangle in triangles {
         for point_index in triangle.0.iter() {
@@ -719,6 +757,7 @@ pub fn get_closest_point_triangle_set<P: PositionProvider>(
     closest_index
 }
 
+#[derive(Debug, PartialEq)]
 pub struct SmoothAngle {
     /// Current angle in radians.
     pub angle: f32,
@@ -765,7 +804,7 @@ impl SmoothAngle {
     }
 
     pub fn at_target(&self) -> bool {
-        (self.target - self.angle).abs() <= std::f32::EPSILON
+        (self.target - self.angle).abs() <= f32::EPSILON
     }
 
     pub fn distance(&self) -> f32 {
